# Schema Organization

The data in Articonf is organized in Use-Cases, Tables, Layers and Nodes. That way, a clear seperation of schema information and actual application data is ensured.

# Trace

A trace is an incoming piece of data. It is a JSON encoded object that can be nested arbitrarily deep. This raw datapoint is the input in Articonf.

__Example:__ A running example will be a pizza shop, that provides its data to Articonf. The shop serves different pizzas and offers a loyalty program where he needs to protocol customers at every visit.

A trace for the shop could look like this: 

    { 
        "type": "blockchain-transaction",
        "content": {
            "ApplicationType": "pizzashop",
            "docType": "pizza",
            "name": "Margherita",
            "description": "A pizza containing mozzarella, basil and tomato sauce.",
            "dough": {
                "type": "wheat",
                "cheese": False
            },
            "sauces": [
                "tomato",
                "chilli oil"
            ]
        }
    }

# Use-Cases

Each trace is assigned to exactly one Use-Case, a label which assigns the piece of data to a Third-Party data source. 

__Example:__ To label each datum, the Use-Case "pizzashop" is added to the system, and each trace has to contain this string identifier with the key ```ApplicationType``` (see example in "Traces").

# Tables

Tables describe how the incoming data is built up and how it is flattened for further processing. One table describes one kind of incoming datum. The incoming JSON data can be nested arbitrarily deep, the job of the table is to *flatten* it in a way, that the data is no longer nested. Each trace has to contain the string identifier of the table with the key ```docType``` (see example in "Traces").

The flattening of a trace with a table results in an object that is no longer nested and that does not contain any lists. Each table must also contain a mapping with the key ```UniqueID```. This property is later used to uniquely identify a trace. The value obtained in this field is [hashed](https://en.wikipedia.org/wiki/Hash_function) afterwards, resulting in an seemingly random string that identifies a trace.

## Flattening Language

To describe the flattening of nested JSON data, a simple syntax was implemented comprising of the following operators:

- __//__ ... With the // operator, a path through a nested object can be described. ```"doughType": "dough//type"``` means that the flattened object will contain a property called "doughType" which gets filled with the value ```type``` of the ```dough``` object in the trace.

- __+__ ... The + Operator concatenates values from the original trace. The datatype of a property including the + operator will automatically become ```String```. ```"fullName": "name+description"``` means that the flattened object will contain a property called ```fullName``` which is the concatenation of the values ```name``` and ```description``` in the trace.

- __[0]__ ... The index operator is used to access an element in a list. ```"firstSauce": "sauces[0]"``` means that the flattened object will contain a property called ```firstSauce``` which is the first entry of the list ```sauces``` in the trace. The ```0``` here can be replaced with any non-negative integer.

__Example:__ The pizza shop has two data kinds: The pizza orders and customer visits. So the shop will add 2 tables, which are called "pizza" and "customer".

The table could look like this:

    {
        "use_case": "pizzashop",
        "name": "pizza",
        "mappings": {
            "UniqueID": "name",
            "name": "name",
            "doughType": "dough//type",
            "fullName": "name+description",
            "firstSauce": "sauces[0]"
        }
    }

The properties ```use_case``` and ```name``` indicate the Use-Case the table belongs to and the tables name. ```mappings``` is the map that indicates how the flattened object is constructed out of the trace, in this example the resulting object will have 4 properties.

# Layers

Once the data has been added to Articonf, the layers take *slices* out of the flattened traces. This is a preproccessing step needed in order to perform the clustering later. Each layers holds two lists: ```properties``` and ```cluster_properties```. The first one selects a set of properties from the underlying table which are the properties stored in the layer while the latter one  marks a subset of these properties as *properties to be clustered after* later on. 

__Example:__ For the ```pizza``` table, a layer was defined which looks like this:

    {
        "use_case": "pizzashop",
        "table": "pizza",
        "name": "name_layer",
        "properties": [
            "UniqueID",
            "name",
        ],
        "cluster_properties": [
            "name"
        ],
    }

Here, ```UniqueID``` and ```name``` were selected as attributes for the layer, but only ```name``` is a cluster attribute.

# Nodes

Finally, a node is a data point which belongs to a layer. It contains only the data selected in the layer description. This is the most fine-grained result of the initial trace.
